package de.unikoblenz.fgbks.core.dmn.verification.verifier;

import de.unikoblenz.fgbks.core.dmn.utils.DmnObjectContainer;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerificationResultEntryFactory;
import de.unikoblenz.fgbks.core.dmn.verification.result.VerifierResult;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.config.AbstractDmnVerifierConfig;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.config.DefaultConfiguration;
import de.unikoblenz.fgbks.core.dmn.verification.verifier.types.VerificationType;
import java.lang.reflect.InvocationTargetException;
import java.util.Objects;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import org.apache.commons.lang3.Validate;
import org.apache.commons.lang3.time.StopWatch;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public abstract class AbstractVerifier implements Verifier {

  protected final Logger LOGGER = LoggerFactory.getLogger(getClass().getSimpleName());

  protected VerificationType verificationType;
  protected AbstractDmnVerifierConfig verifierConfig;
  protected VerifierResult.Builder resultBuilder;
  protected VerificationResultEntryFactory vreFactory;
  protected DmnObjectContainer dmnObjectContainer;

  private long executionTime = -1;

  public VerificationType getVerificationType() {
    return verificationType;
  }

  public final AbstractVerifier withConfiguration(AbstractDmnVerifierConfig configuration) {
    this.verifierConfig = Validate.notNull(configuration);
    this.dmnObjectContainer =
        this.verifierConfig.getConfiguration(DefaultConfiguration.KEY_DMN_OBJECT_CONTAINER);
    return this;
  }

  protected AbstractVerifier() {
    try {
      this.verificationType =
          (VerificationType)
              this.getClass()
                  .getAnnotation(DmnVerifier.class)
                  .verifierType()
                  .getMethod("getInstance")
                  .invoke(null, null);
    } catch (IllegalAccessException | NoSuchMethodException | InvocationTargetException e) {
      e.printStackTrace();
    }
    resultBuilder = VerifierResult.getBuilder().fromVerifier(this);
    vreFactory = VerificationResultEntryFactory.create(resultBuilder);
  }

  public final Future<VerifierResult> verify(ExecutorService executor) {
    Validate.notNull(verifierConfig, "Validation config has to been set.");
    Validate.notNull(executor);
    return executor.submit((Callable<VerifierResult>) this::verify);
  }

  private VerifierResult verify() {
    LOGGER.info("Starting verification " + getVerificationType().getName());
    StopWatch watch = new StopWatch();
    watch.start();
    doVerification();
    watch.stop();
    executionTime = watch.getTime();
    LOGGER.info(
        "Verification "
            + getVerificationType().getName()
            + " finished. "
            + "Time Elapsed: "
            + executionTime
            + " ms");
    return resultBuilder.withExecutionTime(executionTime).build();
  }

  protected abstract void doVerification();

  @Override
  public long getExecutionTime() {
    if (executionTime == -1) {
      throw new IllegalStateException(
          "Verifier must be executed before accessing the execution time!");
    }
    return executionTime;
  }

  @Override
  public boolean equals(Object o) {
    if (this == o) {
      return true;
    }
    if (o == null || getClass() != o.getClass()) {
      return false;
    }
    AbstractVerifier that = (AbstractVerifier) o;
    return verificationType == that.verificationType;
  }

  @Override
  public int hashCode() {
    return Objects.hash(verificationType);
  }
}
